# This is the shared configuration
#
# Options can be found here:
# https://search.nixos.org/options

{ config, pkgs, ... }:

{
  #
  ### Nix Configuration
  #

  # Enable flakes
  nix = {
    package = pkgs.nixVersions.latest;
    extraOptions = ''
      experimental-features = nix-command flakes
    '';
    settings = {
      trusted-users = [
        "root"
        "insert"
      ];
    };
  };

  # Allow unfree packages
  nixpkgs.config.allowUnfree = true;

  # Additional packages
  nixpkgs.overlays = [ (import ./overlay.nix) ];

  #
  ### System Configuration
  #

  # Configure bootloader
  boot.loader.systemd-boot.enable = true;
  boot.loader.efi.canTouchEfiVariables = true;

  # Enable networking and disable firewall
  networking.networkmanager.enable = true;
  networking.firewall.enable = false;

  # Allow SSH connections to device
  services.openssh.enable = true;
  services.openssh.settings.PasswordAuthentication = false;

  # Select internationalisation properties
  i18n.defaultLocale = "en_GB.UTF-8";

  i18n.extraLocaleSettings = {
    LC_ADDRESS = "en_GB.UTF-8";
    LC_IDENTIFICATION = "en_GB.UTF-8";
    LC_MEASUREMENT = "en_GB.UTF-8";
    LC_MONETARY = "en_GB.UTF-8";
    LC_NAME = "en_GB.UTF-8";
    LC_NUMERIC = "en_GB.UTF-8";
    LC_PAPER = "en_GB.UTF-8";
    LC_TELEPHONE = "en_GB.UTF-8";
    LC_TIME = "en_GB.UTF-8";
  };

  # Configure desktop environment
  services.desktopManager.plasma6.enable = true;
  services.displayManager.sddm.enable = true;
  services.displayManager.sddm.wayland.enable = true;
  services.xserver.enable = true;

  # Configure keymap in X11
  services.xserver = {
    xkb = {
      layout = "gb";
      variant = "";
    };
  };

  # Configure console keymap
  console.keyMap = "uk";

  # Install global packages
  environment.systemPackages = with pkgs; [
    wget
    htop
    devenv
    direnv
    nixfmt-rfc-style

    pkgsi686Linux.gperftools # required for Team Fortress 2
    dotnet-sdk # required for Terraria
  ];

  # Enable flatpak
  services.flatpak.enable = true;

  #
  ### User Configuration
  #

  # Define a user account. Don't forget to set a password with ‘passwd’.
  users.users.insert = {
    isNormalUser = true;
    description = "izzy";

    shell = pkgs.fish;
    extraGroups = [
      "networkmanager"
      "wheel"
      "libvirtd"
      "docker"
      "dialout"
      "adbusers"
    ];
  };

  # Enable automatic login for the user.
  services.displayManager.autoLogin.enable = true;
  services.displayManager.autoLogin.user = "insert";

  # Configure shells and shell packages
  environment.shells = with pkgs; [ fish ];
  programs.zsh.enable = true;
  programs.git.enable = true;
  programs.fish.enable = true;

  #
  ### System Services
  #

  # Enable Tailscale
  services.tailscale.enable = true;

  # Enable virtualisation and Docker
  virtualisation.libvirtd.enable = true;
  virtualisation.docker.enable = true;
  virtualisation.waydroid.enable = true;
  virtualisation.spiceUSBRedirection.enable = true;

  # Configure Gnupg for signing
  programs.gnupg.agent = {
    enable = true;
    enableSSHSupport = true;
    pinentryPackage = pkgs.pinentry-qt;
  };

  #
  ### Global Applications
  #

  # Configure Steam
  programs.steam = {
    enable = true;
    remotePlay.openFirewall = true; # Open ports in the firewall for Steam Remote Play
    dedicatedServer.openFirewall = true; # Open ports in the firewall for Source Dedicated Server
  };

  # Configure ADB
  programs.adb.enable = true;

  # Configure NoiseTorch
  programs.noisetorch.enable = true;

  #
  ### Peripherals
  #

  # Enable Wacom drawing tablets
  services.xserver.wacom.enable = true;

  # Enable CUPS to print documents.
  services.printing.enable = true;
}
